// qclistdir.c - A simple 'C' directory listing example
// Original author: Robert Kiraly (OldCoder)
// License:  MIT/X
// Revision: 131013

//--------------------------------------------------------------------
// Overview.

// This is  a  sample Linux 'C' program that checks a directory speci-
// fied on the command line.

// If the directory contains an "index.html" file,  the program simply
// reports this. Otherwise, the program displays a  directory  listing
// in HTML format.

// The program was written for somebody who wanted to  extend a  basic
// webserver.  Specifically,  they  needed  to see how 'C' code  might
// check for the presence of an "index.html" file in a specified  dir-
// ectory and produce a listing  of the directory if the file  was not
// present.

//--------------------------------------------------------------------
// Instructions.

// To compile this program under Linux:
//
// cc -o qclistdir qclistdir.c
// strip qclistdir

// Usage (specify any directory instead of "/tmp"):
//
// qclistdir "/tmp"

// Note: You may need to use "./qclistdir" instead of "qclistdir".

//--------------------------------------------------------------------
// Standard header files.

#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

//--------------------------------------------------------------------
// Basic constants.

#define ZERO        0
#define ONE         1
#define TWO         2

//--------------------------------------------------------------------
// Support routine.

// This  routine takes one argument,  which should specify a directory
// path as an argument.  Note: A directory path in the  current  Linux
// filesystem, as opposed to a web server document path.

// This routine  proceeds as  explained in the Overview.  Normally, it
// returns zero.

// If the directory path is invalid, or if  other errors  are detected
// at this level, this routine returns -1.  In this case, nothing else
// is done.

int qclistdir (char *dirpath)
{

//--------------------------------------------------------------------
// These are the variables that the routine will use.

    char *buf;                  // Points to a buffer
    int n;                      // Holds an integer
    struct stat sbuf;           // Used by a "stat" call

    DIR *dirptr;                // Used to traverse a directory
    struct dirent *entry;       // Points to data about  one thing in-
                                // side a directory
    char *dename;               // Points to the name of one thing in-
                                // side a directory

//--------------------------------------------------------------------
// Some initial sanity checks.

                                // Is the  specified  pathname  either
                                // null or empty?
    if ((dirpath == NULL) || !*dirpath)
    {                           // Yes - This is an error
        return -1;              // For now, just quit in this case
    }
                                // Is the pathname a directory?
    if ((stat (dirpath, &sbuf) != ZERO) || !S_ISDIR (sbuf.st_mode))
    {                           // No  - This is an error
        return -1;              // For now, just quit in this case
    }

//--------------------------------------------------------------------
// Make a pathname for "index.html".

// We are going to check to see if "index.html" is present in the spe-
// cified directory. To do so, we will  need  a full pathname for  the
// file. This code constructs the pathname.

#define LET_THERE_BE_SLACK  25

    n   = strlen (dirpath);     // Length of directory pathname
                                // Make a buffer that is  large enough
                                // to hold the pathname with  "/index.
                                // html" added
    buf = malloc (n + LET_THERE_BE_SLACK);

    if (buf == NULL)            // Did the buffer get created?
    {                           // No  - This is an error
        return;                 // For now, just quit in this case
    }
                                // Put the pathname plus "/index.html"
                                // in the new buffer
    strcpy (buf, dirpath);
    strcat (buf, "/index.html");

//--------------------------------------------------------------------
// The main enchilada.

                                // Is "index.html" in the directory?
    if (stat (buf, &sbuf) == ZERO)
    {                           // Yes - "index.html" is there
                                // Do  whatever  should  be  done with
                                // "index.html"
        puts ("I see index.html there");
    }
    else
    {                           // No  - "index.html" is not there
                                // Let's produce a directory listing

                                // Open the directory for reading
        if ((dirptr = opendir (dirpath)) != NULL)
        {                       // The open was successful
                                // Output an HTML header
            printf ("%s", "<html>\n<body>\n");

                                // Loop through all the entries
            while ((entry = readdir (dirptr)) != NULL)
            {                   // Process the next entry
                                // Point to its name
                dename = entry->d_name;
                                // For now,  skip entries whose  names
                                // start with a period. These are con-
                                // sidered to be special cases.
                if (*dename == '.') continue;
                                // Output the entry.  Note: In the fu-
                                // ture, a link might be generated for
                                // subdirectory entries.
                printf ("<p>%s</p>\n", dename);
            }
                                // Output an HTML trailer
            printf ("%s", "</body>\n</html>\n");

            closedir (dirptr);  // We are done with the directory.  So
                                // close it.
        }
    }

    free (buf);                 // Release the buffer
    return ZERO;                // Normal exit
}

//--------------------------------------------------------------------
// This is the main program.

void main (int argc, char **argv)
{
    int n;                      // Holds an integer

    if (argc != TWO)            // Is argument count correct?
    {                           // No  - Error
                                // Print usage text
        puts ("Usage: qclistdir \"/tmp\" or some other directory");
        exit (ONE);             // Error exit
    }

    n = qclistdir (argv [ONE]); // Pass argument to support routine
    exit (n ? ONE : ZERO);      // Exit with appropriate status
}
